<?php
/**
 * @package  WooadvaricsPlugin
 */
/*
Plugin Name: Adiabax Wooadvarics Plugin
Plugin URI: http://adiabax.it/wooadvaricsplugin
Description: Extends Woocommerce to integrate Advarics ERP System
Version: 1.9.1
Author: Adiabax Srl
Author URI: http://adiabax.it
License: GPLv2 or later
Text Domain: wooadvarics-plugin
Domain Path: /languages
*/

defined( 'ABSPATH' ) or die( 'Hey, what are you doing here? You silly human!' );
// Define CONSTANTS
define( 'ADBX_PLUGIN_PATH', plugin_dir_path( __FILE__ ) );
define( 'ADBX_PLUGIN_URL', plugin_dir_url( __FILE__ ) );
define( 'ADBX_PLUGIN', plugin_basename( __FILE__ ) );

require ADBX_PLUGIN_PATH . 'inc/wooadvarics-plugin-config.php';
require ADBX_PLUGIN_PATH . 'inc/wooadvarics-plugin-utility.php';
require ADBX_PLUGIN_PATH . 'inc/wooadvarics-plugin-settings.php';

require ADBX_PLUGIN_PATH . 'inc/wooadvarics-plugin-synchronize.php';
require ADBX_PLUGIN_PATH . 'inc/wooadvarics-plugin-connect.php';


global $taxonomy_attributes;
$taxonomy_attributes = array();

if ( !class_exists( 'WooadvaricsPlugin' ) ) {
	
	class WooadvaricsPlugin
	{
		
		public $plugin;	
		
		public $settings;
		
		public $wooadvarics_options;
		
		function __construct() {
			
			$this->plugin = plugin_basename( __FILE__ );
			
			//add_action( 'init', array( $this, 'wpdocs_load_textdomain' ) );
			$this->wpdocs_load_textdomain();
			
			add_action( 'admin_enqueue_scripts', array( $this, 'enqueue' ) );

			$this->settings = new WooAdvaricsSettingsApi();
			
			$this->settings->register();
			
			add_action( 'woocommerce_thankyou', array( $this, 'adbxNewOrderInsert' ), 9 );
			
			add_action( 'woocommerce_order_actions', array( $this, 'adbxOrderStornoAction' ), 9 );
			add_action( 'woocommerce_order_action_adbx_storno_order', array( $this, 'adbxAdvaricsOrderStorno' ), 9 );
			
			add_action( 'woocommerce_order_actions', array( $this, 'adbxOrderPositionStornoAction' ), 9 );
			add_action( 'woocommerce_order_action_adbx_storno_orderposition', array( $this, 'adbxAdvaricsOrderPositionStorno' ), 9 );

			
			add_action( 'woocommerce_order_actions', array( $this, 'adbxOrderManualSendAction' ), 9 );
			add_action( 'woocommerce_order_action_adbx_send_order', array( $this, 'adbxAdvaricsSendOrder' ), 9 );
			
			add_filter('cron_schedules', array( $this, 'wooadvaricsCronSchedules' ));
			add_action( 'wooadvarics_cron_hook', array( $this, 'adbxScheduledSynch' ) );

			add_action('wooadvarics_process_load',array( $this, 'adbxProcessLoad' ), 10, 3);
			
			$synch_option = get_option( 'adbx_advarics_webshopsynch' );
			if ( $synch_option > 0 )
			{
				if ( ! wp_next_scheduled( 'wooadvarics_cron_hook' ) ) 
				{
					switch ( $synch_option )
					{
						case 1: 
								wp_schedule_event( time(), '5min', 'wooadvarics_cron_hook' );
								break;
						case 2: 
								wp_schedule_event( time(), '15min', 'wooadvarics_cron_hook' );
								break;
						case 3: 
								wp_schedule_event( time(), '30min', 'wooadvarics_cron_hook' );
								break;
						case 4: 
								wp_schedule_event( time(), 'hourly', 'wooadvarics_cron_hook' );
								break;
						case 5: 
								wp_schedule_event( time(), '180min', 'wooadvarics_cron_hook' );
								break;
						case 6: 
								wp_schedule_event( time(), 'twicedaily', 'wooadvarics_cron_hook' );
								break;
						case 7: 
								wp_schedule_event( time(), 'daily', 'wooadvarics_cron_hook' );
								break;
						case 8: 
								wp_schedule_event( time(), 'weekly', 'wooadvarics_cron_hook' );
								break;
						default:	
								wp_schedule_event( time(), '30min', 'wooadvarics_cron_hook' );
								break;					
					}
				}
			}	
			else
			{
				if ( wp_next_scheduled( 'wooadvarics_cron_hook' ) ) 
				{
					$timestamp = wp_next_scheduled( 'wooadvarics_cron_hook' );
					wp_unschedule_event( $timestamp, 'wooadvarics_cron_hook' );
				}
			}	
			
			//Perfect Brands customization	
			if ($active_plugins = get_option('active_plugins', array())) {
				$adbx_pwb = false;
				foreach ($active_plugins as $key => $active_plugin) {
					if ($active_plugin == 'perfect-woocommerce-brands/perfect-woocommerce-brands.php')  
						$adbx_pwb = true;
				}
				update_option('adbx_pwb_active', $adbx_pwb);
			} 
			
			
		}
		
		public function enqueue() {

			wp_enqueue_style( 'wooadvaricspluginstyle', plugins_url( '/assets/wooadvaricsstyle.css', __FILE__ ) );
			wp_enqueue_script( 'wooadvaricspluginscript', plugins_url( '/assets/wooadvaricsscript.js', __FILE__ ) );
		
		}
		

		function activate() {

				require_once plugin_dir_path( __FILE__ ) . 'inc/wooadvarics-plugin-activate.php';
				WooadvaricsPluginActivate::activate();
		}
		

		function deactivate() {

				require_once plugin_dir_path( __FILE__ ) . 'inc/wooadvarics-plugin-deactivate.php';
				WooadvaricsPluginDeactivate::deactivate();
		}
		

		function wpdocs_load_textdomain() {
		  load_plugin_textdomain( 'wooadvarics-plugin', false, dirname( plugin_basename( __FILE__ ) ) . '/languages/' );
		}
					
		
		function adbxNewOrderInsert( $order_id ) {
			

			
			if ( get_option('adbx_webshop_order_destination') == 1 )
			{
				require_once plugin_dir_path( __FILE__ ) . 'inc/wooadvarics-plugin-neworder.php';
				$order = new WC_Order( $order_id );
				$orderstat = $order->get_status();
				$paymethod = $order->get_payment_method_title();
				Logger::adbxWriteLog('New order with wooID:' . $order_id . ', ordere status:'  .  $orderstat . ', payment method:' . $paymethod);
				$new_order = new AdvaricsSynchOrder();
				$woo_num_order = $new_order->adbxGetWooOrder( $order_id );
				$json_response = $new_order->adbxCreateErpOrder();
				
				$json_data_array = json_decode( $json_response, true);
				/*
				if (($json_data_array["success"] == true))
				{
					$message = sprintf( __( 'Order sent to Advarics .', 'my-textdomain' ), wp_get_current_user()->display_name );
					$order->add_order_note( $message );
				}
				else
				{
					$message = sprintf( __( 'ERROR, order not sent to Advarics, Try manually.', 'my-textdomain' ), wp_get_current_user()->display_name );
					$order->add_order_note( $message );
				}
				*/
				$order->add_order_note( $json_data_array["message"] );
				Logger::adbxWriteLog( $json_data_array["message"] );
			}	

		}
		
		
		public function adbxProcessLoad( $init_blocco, $end_blocco, $contents){

			//$synch_instance = new AdbxWooProductsUpdate();
			//$synch_instance->adbxExecUpdate( $init_blocco, $end_blocco, $contents );
			$last_blocco = $end_blocco-1;
			//Logger::adbxWriteLog( "Init block: $init_blocco - $last_blocco");
			$synch_instance = new AdbxWooProductsUpdate();
			$synch_instance->adbxExecUpdate( $init_blocco, $end_blocco, $contents );
			//Logger::adbxWriteLog( "End block: $init_blocco - $last_blocco");
		}
		
		
		function adbxScheduledSynch() {

			$synch_type 	= get_option('adbx_advarics_changedat_synch');
			
			$conn_instance 	= new AdbxSynchWooAdvarics();
			//$synch_json		= $conn_instance->adbxExecSynchronization(1);
			$synch_json		= $conn_instance->adbxExecSynchronization($synch_type);

			if ($synch_json)
			{	
				$array_request	= json_decode($conn_instance->erp_json_response, true);
				$num_record 	= count($array_request["articles"]);
				$this->adbxProcessLoad( 0, $num_record, $conn_instance->erp_json_response);
			}	
			
		}
		
		function adbxOrderStornoAction( $actions ) {

			global $theorder;

			$advarics_order_stauts = get_post_meta( $theorder->get_id(), 'adbx_sent_to_erp', true );
			//if ( ( $theorder->get_status() == 'cancelled' ) OR ( empty($advarics_order_stauts)) ){
			if ( ( $advarics_order_stauts == 2 ) OR ( empty($advarics_order_stauts)) ){
				return $actions;
			}

			$actions['adbx_storno_order'] = __( 'Cancel (Storno) Advarics order', 'my-textdomain' );
			return $actions;
		}
		
		
		function adbxAdvaricsOrderStorno( $order ) {

			require_once plugin_dir_path( __FILE__ ) . 'inc/wooadvarics-plugin-neworder.php';
			
			if (!empty($order)) {
				$woo_order = new AdvaricsSynchOrder();
				$json_response = $woo_order->adbxCancelOrder( $order );
				
				$json_data_array = json_decode( $json_response, true);
				if (($json_data_array["success"] == true))
				{
					$message = sprintf( __( 'Order cancelled by %s .', 'my-textdomain' ), wp_get_current_user()->display_name );
					$order->add_order_note( $message );
				}
				else
				{
					$message = sprintf( __( 'Error cancelling order in Advarics by %s.', 'my-textdomain' ), wp_get_current_user()->display_name );
					$order->add_order_note( $message );
				}
				
				Logger::adbxWriteLog( $json_data_array["message"] );

			}

		}
		
		function adbxOrderPositionStornoAction( $actions ) {

			global $theorder;
			$items 	= $theorder->get_items();
			$num_refunded_items = 0;
			
			foreach ( $theorder->get_items() as $item_id => $item ) 
			{
			   $item_qty_original	= $item->get_quantity();
			   $item_qty_refunded 	= $theorder->get_qty_refunded_for_item( $item_id );
			   $new_qty 			= $item_qty_original + $item_qty_refunded;
			   
			   if ($new_qty == 0)
				   $num_refunded_items++;
			   
			   //Logger::adbxWriteLog(', qty_refunded: ' . $item_qty_refunded. ' -- ' . $item_id . ' -- orign_qty:' .  $item_qty_original . ', status:' . $new_qty );
			}

			$advarics_order_stauts = get_post_meta( $theorder->get_id(), 'adbx_sent_to_erp', true );
			//if ( ( $theorder->get_status() == 'cancelled' ) OR ( empty($advarics_order_stauts)) OR $num_refunded_items == 0){
			if ( ( $advarics_order_stauts >= 2 ) OR ( empty($advarics_order_stauts)) OR $num_refunded_items == 0){
				return $actions;
			}

			$actions['adbx_storno_orderposition'] = __( 'Cancel (Storno) Advarics order Position', 'my-textdomain' );
			return $actions;
		}
		
		
		function adbxAdvaricsOrderPositionStorno( $order ) {

			require_once plugin_dir_path( __FILE__ ) . 'inc/wooadvarics-plugin-neworder.php';
			
			if (!empty($order)) 
			{
				$order_nr	= $order->get_id();
				$items 		= $order->get_items();
				$pos = 1;
				$woo_order 	= new AdvaricsSynchOrder();

				foreach ( $order->get_items() as $item_id => $item ) 
				{
				   $item_qty_original	= $item->get_quantity();
				   $item_qty_refunded 	= $order->get_qty_refunded_for_item( $item_id );
				   $new_qty 			= $item_qty_original + $item_qty_refunded;
				   
				   if ($new_qty == 0)
				   {
					    //$status_position = 'cancelled';
						
						$json_response = $woo_order->adbxCancelOrderPosition( $order, $pos );
						
						$json_data_array = json_decode( $json_response, true);
						if (($json_data_array["success"] == true))
						{
							$message = sprintf( __( 'Order position nr.' . $pos . ' cancelled by %s .', 'my-textdomain' ), wp_get_current_user()->display_name );
							$order->add_order_note( $message );
						}
						else
						{
							$message = sprintf( __( 'Error cancelling orderposition in Advarics by %s.', 'my-textdomain' ), wp_get_current_user()->display_name );
							$order->add_order_note( $message );
						}
						
						Logger::adbxWriteLog( 'Order_nr:' . $order_nr . ', ' . $message );
				   }
				  
				   /*else
					   $status_position = 'confirmed';
				   */
				   
					$pos++;
				}
			}
			
		}
		

		
		function adbxOrderManualSendAction( $actions ) {

			global $theorder;

			$order_status = $theorder->get_status();
			$advarics_order_stauts = get_post_meta( $theorder->get_id(), 'adbx_sent_to_erp', true );
			
			//if ( (( $order_status <> 'processing') AND ($order_status <> 'completed') AND  ($order_status <> 'on-hold') AND ($order_status <> 'pending')) OR ( !empty($advarics_order_stauts)) )
			if ( (( $order_status <> 'refunded') AND ( $order_status <> 'processing') AND ($order_status <> 'completed') AND  ($order_status <> 'on-hold') AND ($order_status <> 'pending')) OR ( !empty($advarics_order_stauts)) )
				return $actions;
			else
			{
				$actions['adbx_send_order'] = __( 'Send order to Advarics', 'my-textdomain' );
				return $actions;	
			}	


		}

		
		function adbxAdvaricsSendOrder( $order ) {

			require_once plugin_dir_path( __FILE__ ) . 'inc/wooadvarics-plugin-neworder.php';
			
			$order_status 	= $order->get_status();
			if ( ($order_status == 'processing') OR ($order_status == 'refunded') OR ($order_status == 'on-hold') OR ($order_status == 'pending') OR ($order_status == 'completed'))
			{
				if (!empty($order)) {
					$new_order = new AdvaricsSynchOrder();
					$woo_num_order = $new_order->adbxGetWooOrder( $order->id );
					$json_response = $new_order->adbxCreateErpOrder();
					
					$json_data_array = json_decode( $json_response, true);
					if (($json_data_array["success"] == true))
					{
						$message = sprintf( __( 'Order sent to Advarics by %s .', 'my-textdomain' ), wp_get_current_user()->display_name );
						$order->add_order_note( $message );
					}
					else
					{
						$message = $json_data_array["message"];
						$order->add_order_note( $message );
					}
					
					Logger::adbxWriteLog( $json_data_array["message"] );
					
				}
			}	

		}
		
		
		function wooadvaricsCronSchedules($schedules){
			if(!isset($schedules["5min"])){
				$schedules["5min"] = array(
					'interval' => 5*60,
					'display' => __('Once every 5 minutes'));
			}
			if(!isset($schedules["15min"])){
				$schedules["15min"] = array(
					'interval' => 15*60,
					'display' => __('Once every 15 minutes'));
			}
			if(!isset($schedules["30min"])){
				$schedules["30min"] = array(
					'interval' => 30*60,
					'display' => __('Once every 30 minutes'));
			}
			if(!isset($schedules["180min"])){
				$schedules["180min"] = array(
					'interval' => 180*60,
					'display' => __('Once every 3 hours'));
			}

			return $schedules;
		}
	
	}

	
	$wooadvaricsPlugin = new WooadvaricsPlugin();

	register_activation_hook( __FILE__, array( $wooadvaricsPlugin, 'activate' ) );

	register_deactivation_hook( __FILE__, array( $wooadvaricsPlugin, 'deactivate' ) );
	
	//update_option( 'adbx_simple_product', '' );
	
}


/*
class Logger {
	
	static function adbxWriteLog( $logstring ){	

		$log = get_option( 'adbx_advarics_log' );
		if ($log == 1)
		{
			$logfile = fopen(ADBX_LOG_FILE, 'a');  
			fwrite($logfile, "\n" . date('d-m-Y H:i:s') . ' ->' . $logstring);
			fclose($logfile); 	
		}
	}
	
}
*/

/*
function adbx_python_crypt( $string, $action, $e_method ) {

	$secret_key	= SKEY;
	$secret_iv	= SIV;
	
    $output = false;
	switch ($e_method) {
		case 0: $encrypt_method = "AES-256-CBC";
				break;
		case 1: $encrypt_method = "AES-128-CBC";
				break;
		default:		
				$encrypt_method = "AES-128-CBC";
				break;
	}
	
    if( $action == 'e' ) {
		$output = base64_encode( openssl_encrypt( $string, $encrypt_method, $secret_key,OPENSSL_RAW_DATA, $secret_iv ) );

    }
    else if( $action == 'd' ){
		$output = openssl_decrypt( base64_decode( $string ), $encrypt_method, $secret_key, OPENSSL_RAW_DATA, $secret_iv );
    }
    return $output;
}
*/


require ADBX_PLUGIN_PATH . 'plugin-update-checker/plugin-update-checker.php';
$myUpdateChecker = Puc_v4_Factory::buildUpdateChecker(
	'https://www.adiabax.it/ea/wooadvarics/details.json',
	__FILE__, //Full path to the main plugin file or functions.php.
	'unique-plugin-or-theme-slug'
);